// Copyright © 2012 Qtrac Ltd.
// 
// This program or package and any associated files are licensed under the
// Apache License, Version 2.0 (the "License"); you may not use these files
// except in compliance with the License. You can get a copy of the License
// at: http://www.apache.org/licenses/LICENSE-2.0.
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
    "fmt"
)

type Painter interface {
    Paint()
}

type Clicker interface {
    Click()
}

type Widget struct {
    X, Y int
}

type Label struct {
    Widget        // Embedding (delegation)
    Text   string // Aggregation
}

func (label Label) Paint() {
    fmt.Printf("%p:Label.Paint(%q)\n", &label, label.Text)
}

type Button struct {
    Label // Embedding (delegation)
}

func NewButton(x, y int, text string) Button {
    return Button{Label{Widget{x, y}, text}}
}

func (button Button) Paint() { // Override
    fmt.Printf("%p:Button.Paint(%q)\n", &button, button.Text)
}

func (button Button) Click() {
    fmt.Printf("%p:Button.Click()\n", &button)
}

type ListBox struct {
    Widget          // Embedding (delegation)
    Texts  []string // Aggregation
    Index  int      // Aggregation
}

func (listBox ListBox) Paint() {
    fmt.Printf("%p:ListBox.Paint(%q)\n", &listBox, listBox.Texts)
}

func (listBox ListBox) Click() {
    fmt.Printf("%p:ListBox.Click()\n", &listBox)
}

func main() {
    label := Label{Widget{10, 10}, "State:"}
    button1 := Button{Label{Widget{10, 70}, "OK"}}
    button2 := NewButton(50, 70, "Cancel")
    fmt.Println(label, button1, button2)
    fmt.Println()
    listBox := ListBox{Widget{10, 40},
        []string{"AL", "AK", "AZ", "AR"}, 0}
    for _, painter := range []Painter{label, listBox, button1, button2} {
        painter.Paint()
    }
    fmt.Println()
    for _, widget := range []interface{}{label, listBox, button1, button2} {
        if clicker, ok := widget.(Clicker); ok {
            clicker.Click()
        }
    }
    fmt.Println()
    /* won't compile because label is not a Clicker
    for _, clicker := range []Clicker{label, listBox,
        button1, button2} {
        clicker.Click()
    }
    */
    clickables := []Clicker{listBox, button1, button2}
    for _, clicker := range clickables {
        clicker.Click()
    }
}
